package org.openxma.dsl.reference.dao.impl;

import org.openxma.dsl.platform.dao.*;
import org.openxma.dsl.platform.dao.impl.*;
import org.openxma.dsl.platform.service.Mapper;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import java.util.Collection;
import java.util.Date;
import org.openxma.dsl.reference.dao.OrderDao;
import org.openxma.dsl.reference.model.Customer;
import org.openxma.dsl.reference.model.Order;
import org.openxma.dsl.reference.model.impl.OrderImpl;
import org.openxma.dsl.reference.types.valueobject.OrderNumber;
/**
 * @see OrderDao
 */
@SuppressWarnings("unused")
public abstract class OrderDaoGenImpl extends GenericDaoHibernateImpl<Order,String> implements OrderDao {

    public OrderDaoGenImpl() {
        super(OrderImpl.class);
    }
    
    /**
     * @param context the context for the object creation
     * @return a transient instance of entity <code>Order</code> with an already assigned identifier.
     */
    public Order createEntity(Object context) {
        return create();
    }

    public Order create()
    {
        Order order = createEntityInstance();
        order.setOid(dataFieldMaxValueIncrementer.nextStringValue());
        return order;
    }

    public Order create(Customer customer,OrderNumber orderNumber,String storeId,Date placementDate,String orderState,Double totalAmount) {
        Assert.notNull(customer, "Parameter 'customer' must not be null");
        Assert.notNull(orderNumber, "Parameter 'orderNumber' must not be null");
        Assert.notNull(storeId, "Parameter 'storeId' must not be null");
        Assert.notNull(placementDate, "Parameter 'placementDate' must not be null");
        Assert.notNull(orderState, "Parameter 'orderState' must not be null");
        Assert.notNull(totalAmount, "Parameter 'totalAmount' must not be null");
        Order order = (Order) create();
        order.setCustomer(customer);
        order.setOrderNumber(orderNumber);
        order.setStoreId(storeId);
        order.setPlacementDate(placementDate);
        order.setOrderState(orderState);
        order.setTotalAmount(totalAmount);
        return order;
    }

    public Order createAndSave(Customer customer,OrderNumber orderNumber,String storeId,Date placementDate,String orderState,Double totalAmount) {
        Order order = create(customer,orderNumber,storeId,placementDate,orderState,totalAmount);
        saveOrUpdate(order);
        return order;
    }
    
    public Order loadByOrderNk(String firstName,String lastName,Date birthDate,String storeId,Date placementDate) {
        Criterion criterion = equal("customer.firstName", firstName, false);
        criterion = and(criterion,"customer.lastName", lastName, false);
        criterion = and(criterion,"customer.birthDate", birthDate, false);
        criterion = and(criterion,"storeId", storeId, false);
        criterion = and(criterion,"placementDate", placementDate, false);
        Criteria criteria = createCriteria(criterion);
        criteria.createAlias("customer","customer");
        return (Order) unique(criteria);
    }

    public Order loadByOrderUkOrderNumber(Customer customer,OrderNumber orderNumber) {
        Criterion criterion = equal("customer", customer, false);
        criterion = and(criterion,"orderNumber", orderNumber, false);
        Criteria criteria = createCriteria(criterion);
        return (Order) unique(criteria);
    }

    public Order loadByOrderUkOrderNumber(String customerOid,OrderNumber orderNumber) {
        Criterion criterion = equal("customer.oid", customerOid, true);
        criterion = and(criterion,"orderNumber", orderNumber, false);
        Criteria criteria = createCriteria(criterion);
        criteria.createAlias("customer","customer");
        return (Order) unique(criteria);
    }
        
    public Collection<Order> findAllByCustomer(Customer customer) {
		String queryName = OrderDao.FIND_ALL_BY_CUSTOMER;
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
        namedQuery.setEntity("customer", customer);
        applyFindAllByCustomerQueryHints(namedQuery,customer);
        return list(namedQuery);
    }

    protected void applyFindAllByCustomerQueryHints(Query query,Customer customer) {
        // override this method and apply other hints that influence how a query is executed
    }
        
    public Collection<Order> findAllByOrderNumber(OrderNumber orderNumber) {
		String queryName = OrderDao.FIND_ALL_BY_ORDER_NUMBER;
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
        namedQuery.setParameter("orderNumber", orderNumber);
        applyFindAllByOrderNumberQueryHints(namedQuery,orderNumber);
        return list(namedQuery);
    }

    protected void applyFindAllByOrderNumberQueryHints(Query query,OrderNumber orderNumber) {
        // override this method and apply other hints that influence how a query is executed
    }
    
    protected Mapper getNamedQueryMapper(String queryName) {
		return new OrderDaoQueryMapper();
	}
	
	protected class OrderDaoQueryMapper extends NamedQueryMapper {
	
		@SuppressWarnings("unchecked")
		public <E> E mapOne(Object sourceObject, E targetObject, Object context) {
			return super.mapOne(sourceObject, targetObject);
		}
	}
}
