package org.openxma.dsl.reference.service.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.openxma.dsl.platform.service.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.openxma.dsl.reference.dao.CategoryDao;
import org.openxma.dsl.reference.dao.CertificateDao;
import org.openxma.dsl.reference.dao.ProductDao;
import org.openxma.dsl.reference.dao.ProductImageDao;
import org.openxma.dsl.reference.dao.SupplierDao;
import org.openxma.dsl.reference.dto.CategoryView;
import org.openxma.dsl.reference.dto.ProductView;
import org.openxma.dsl.reference.model.Category;
import org.openxma.dsl.reference.model.Product;
import org.openxma.dsl.reference.model.Supplier;
import org.openxma.dsl.reference.service.CategoryService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
/**
 *  Service for managing categories.
 */
public abstract class CategoryServiceGenImpl implements CategoryService {
    protected final Log logger = LogFactory.getLog(getClass());
    @Autowired(required=false)
    protected Mapper mapper;
    protected CategoryDao categoryDao;
    protected ProductDao productDao;
    protected SupplierDao supplierDao;
    protected ProductImageDao productImageDao;
    protected CertificateDao certificateDao;
    
    @Autowired
    public void setCategoryDao(CategoryDao categoryDao) {
        this.categoryDao = categoryDao;
    }

    @Autowired
    public void setProductDao(ProductDao productDao) {
        this.productDao = productDao;
    }

    @Autowired
    public void setSupplierDao(SupplierDao supplierDao) {
        this.supplierDao = supplierDao;
    }

    @Autowired
    public void setProductImageDao(ProductImageDao productImageDao) {
        this.productImageDao = productImageDao;
    }

    @Autowired
    public void setCertificateDao(CertificateDao certificateDao) {
        this.certificateDao = certificateDao;
    }
    
    @Transactional
	public CategoryView save(CategoryView categoryView) {
        Assert.notNull(categoryView, "parameter 'categoryView' must not be null");
        Category category = this.mapper.createAndMapOne(categoryView, Category.class,"saveCategory");
        addSubCategories(category,categoryView);
        addProducts(category,categoryView);
        this.categoryDao.saveOrUpdate(category);
        return this.mapper.mapOne(category,new CategoryView());
    }
    
    private void addSubCategories(Category parentCategory,CategoryView categoryView) {
        if (categoryView.getSubCategories() != null) {
            for (CategoryView _categoryView : categoryView.getSubCategories()) {
                Category category = this.mapper.createAndMapOne(_categoryView, Category.class,"saveCategory");
                addProducts(category,_categoryView);
                parentCategory.addSubCategories(category);
            }
        }
    }
    
    private void addProducts(Category parentCategory,CategoryView categoryView) {
        if (categoryView.getProducts() != null) {
            for (ProductView _productView : categoryView.getProducts()) {
                Product product = this.mapper.createAndMapOne(_productView, Product.class,"saveProduct");
                Supplier supplier = this.mapper.createAndMapOne(_productView, Supplier.class,"supplier");
                if (supplier.getOid() != null) {
                    supplier = supplierDao.load(supplier.getOid());
                    product.setSupplier(supplier);
                }
                parentCategory.addProducts(product);
            }
        }
    }
    
    @Transactional(readOnly = true)
	public CategoryView loadCategoryView(Long oid) {
        Assert.notNull(oid, "parameter 'oid' must not be null");
        return this.mapper.createAndMapOne(this.categoryDao.load(oid),CategoryView.class);
    }
    
    @Transactional
	public void update(CategoryView categoryView) {
        Assert.notNull(categoryView, "parameter 'categoryView' must not be null");
        Category mappedCategory = this.mapper.createAndMapOne(categoryView, Category.class);
        Category category = this.categoryDao.load(mappedCategory.getOid());
        this.mapper.mapOne(categoryView, category,"updateCategory");
    }
    
    @Transactional
	public void deleteCategory(Long oid) {
        Assert.notNull(oid, "parameter 'oid' must not be null");
        this.categoryDao.delete(oid);
    }
        
    @Transactional
    public CategoryView saveOrUpdate(CategoryView categoryView) {
        Assert.notNull(categoryView, "parameter 'categoryView' must not be null");
        if ( categoryView.getOid() == null ) {
            return this.save(categoryView);
        }
        else {
            this.update(categoryView);
            return categoryView;
        }
    }

    
}
